from typing import TypedDict, List
from .member import Member
from .emoji import PartialEmoji


class _MessageEventOptional(TypedDict, total=False):
    guild_id: str


class MessageDeleteEvent(_MessageEventOptional):
    id: str
    channel_id: str


class BulkMessageDeleteEvent(_MessageEventOptional):
    ids: List[str]
    channel_id: str


class _ReactionActionEventOptional(TypedDict, total=False):
    guild_id: str
    member: Member


class MessageUpdateEvent(_MessageEventOptional):
    id: str
    channel_id: str


class ReactionActionEvent(_ReactionActionEventOptional):
    user_id: str
    channel_id: str
    message_id: str
    emoji: PartialEmoji


class _ReactionClearEventOptional(TypedDict, total=False):
    guild_id: str


class ReactionClearEvent(_ReactionClearEventOptional):
    channel_id: str
    message_id: str


class _ReactionClearEmojiEventOptional(TypedDict, total=False):
    guild_id: str


class ReactionClearEmojiEvent(_ReactionClearEmojiEventOptional):
    channel_id: int
    message_id: int
    emoji: PartialEmoji


class _IntegrationDeleteEventOptional(TypedDict, total=False):
    application_id: str


class IntegrationDeleteEvent(_IntegrationDeleteEventOptional):
    id: str
    guild_id: str



import itertools
import copy
import functools
import inspect
import re

from typing import Optional, TYPE_CHECKING

import qq.utils

from .core import Group, Command
from .errors import CommandError

if TYPE_CHECKING:
    from .context import Context

__all__ = (
    'Paginator',
    'HelpCommand',
    'DefaultHelpCommand',
    'MinimalHelpCommand',
)


# help -> 在顶部显示机器人的信息并列出子命令
# help 命令 -> 显示命令的详细信息
# help 命令 <子命令链> -> 同上

# <描述>

# <带别名的命令签名>

# <长文档>

# 齿轮:
#   <命令> <简文档>
#   <命令> <简文档>
# 其他齿轮:
#   <命令> <简文档>
# 无类别:
#   <命令> <简文档>

# 键入 <前缀>help 命令以获取有关命令的更多信息。
# 你还可以键入 <前缀>help 类别 以获取有关类别的更多信息。


class Paginator:
    """一个有助于为 qq 消息分页代码块的类。

    .. container:: operations

        .. describe:: len(x)

            返回分页器中的字符总数。

    Attributes
    -----------
    prefix: :class:`str`
        插入到每个页面的前缀。 例如 三个反引号。
    suffix: :class:`str`
        后缀附加在每一页的末尾。 例如 三个反引号。
    max_size: :class:`int`
        页面中允许的最大代码点数。
    linesep: :class:`str`
        行间插入的字符串。 例如 换行符。
    """

    def __init__(self, prefix='', suffix='', max_size=2000, linesep='\n'):
        self.prefix = prefix
        self.suffix = suffix
        self.max_size = max_size
        self.linesep = linesep
        self.clear()

    def clear(self):
        """清除分页器以使其没有页面。"""
        if self.prefix is not None:
            self._current_page = [self.prefix]
            self._count = len(self.prefix) + self._linesep_len  # prefix + newline
        else:
            self._current_page = []
            self._count = 0
        self._pages = []

    @property
    def _prefix_len(self):
        return len(self.prefix) if self.prefix else 0

    @property
    def _suffix_len(self):
        return len(self.suffix) if self.suffix else 0

    @property
    def _linesep_len(self):
        return len(self.linesep)

    def add_line(self, line='', *, empty=False):
        """向当前页面添加一行。

        如果该行超过 :attr:`max_size`，则会引发异常。

        Parameters
        -----------
        line: :class:`str`
            要添加的行。
        empty: :class:`bool`
            指示是否应添加另一个空行。

        Raises
        ------
        RuntimeError
            该行对于当前的 :attr:`max_size` 来说太大了。
        """
        max_page_size = self.max_size - self._prefix_len - self._suffix_len - 2 * self._linesep_len
        if len(line) > max_page_size:
            raise RuntimeError(f'行超过最大页面大小 {max_page_size}')

        if self._count + len(line) + self._linesep_len > self.max_size - self._suffix_len:
            self.close_page()

        self._count += len(line) + self._linesep_len
        self._current_page.append(line)

        if empty:
            self._current_page.append('')
            self._count += self._linesep_len

    def close_page(self):
        """提前终止页面。"""
        if self.suffix is not None:
            self._current_page.append(self.suffix)
        self._pages.append(self.linesep.join(self._current_page))

        if self.prefix is not None:
            self._current_page = [self.prefix]
            self._count = len(self.prefix) + self._linesep_len  # prefix + linesep
        else:
            self._current_page = []
            self._count = 0

    def __len__(self):
        total = sum(len(p) for p in self._pages)
        return total + self._count

    @property
    def pages(self):
        """List[:class:`str`]: 返回呈现的页面列表。"""
        # we have more than just the prefix in our current page
        if len(self._current_page) > (0 if self.prefix is None else 1):
            self.close_page()
        return self._pages

    def __repr__(self):
        fmt = '<Paginator prefix: {0.prefix!r} suffix: {0.suffix!r} linesep: {0.linesep!r} max_size: {0.max_size} ' \
              'count: {0._count}> '
        return fmt.format(self)


def _not_overriden(f):
    f.__help_command_not_overriden__ = True
    return f


class _HelpCommandImpl(Command):
    def __init__(self, inject, *args, **kwargs):
        super().__init__(inject.command_callback, *args, **kwargs)
        self._original = inject
        self._injected = inject

    async def prepare(self, ctx):
        self._injected = injected = self._original.copy()
        injected.context = ctx
        self.callback = injected.command_callback

        on_error = injected.on_help_command_error
        if not hasattr(on_error, '__help_command_not_overriden__'):
            if self.cog is not None:
                self.on_error = self._on_error_cog_implementation
            else:
                self.on_error = on_error

        await super().prepare(ctx)

    async def _parse_arguments(self, ctx):
        # Make the parser think we don't have a cog so it doesn't
        # inject the parameter into `ctx.args`.
        original_cog = self.cog
        self.cog = None
        try:
            await super()._parse_arguments(ctx)
        finally:
            self.cog = original_cog

    async def _on_error_cog_implementation(self, dummy, ctx, error):
        await self._injected.on_help_command_error(ctx, error)

    @property
    def clean_params(self):
        result = self.params.copy()
        try:
            del result[next(iter(result))]
        except StopIteration:
            raise ValueError('缺少 context 参数') from None
        else:
            return result

    def _inject_into_cog(self, cog):
        # Warning: hacky

        # Make the cog think that get_commands returns this command
        # as well if we inject it without modifying __cog_commands__
        # since that's used for the injection and ejection of cogs.
        def wrapped_get_commands(*, _original=cog.get_commands):
            ret = _original()
            ret.append(self)
            return ret

        # Ditto here
        def wrapped_walk_commands(*, _original=cog.walk_commands):
            yield from _original()
            yield self

        functools.update_wrapper(wrapped_get_commands, cog.get_commands)
        functools.update_wrapper(wrapped_walk_commands, cog.walk_commands)
        cog.get_commands = wrapped_get_commands
        cog.walk_commands = wrapped_walk_commands
        self.cog = cog

    def _eject_cog(self):
        if self.cog is None:
            return

        # revert back into their original methods
        cog = self.cog
        cog.get_commands = cog.get_commands.__wrapped__
        cog.walk_commands = cog.walk_commands.__wrapped__
        self.cog = None


class HelpCommand:
    r"""帮助命令格式的基本实现。

    .. note::

        每次调用命令本身时，都会在内部深度复制此类的实例，

        这意味着在命令调用之间依赖于此类的状态相同将不会按预期工作。

    Attributes
    ------------
    context: Optional[:class:`Context`]
        调用此帮助格式化程序的 context 。 这通常在分配的帮助命令 :func:`command_callback`\ 被调用后设置。
    show_hidden: :class:`bool`
        指定是否应在输出中显示隐藏命令。
        默认为 ``False`` 。
    verify_checks: Optional[:class:`bool`]
        指定命令是否应该调用和验证它们的 :attr:`.Command.checks`。 如果 ``True``，则总是调用 :attr:`.Command.checks`。
        如果 ``None`` ，则仅在公会设置中调用 :attr:`.Command.checks`。 如果 ``False``，则从不调用 :attr:`.Command.checks`。
        默认为 ``True`` 。

    command_attrs: :class:`dict`
        用于构建帮助命令的选项字典。
        这允许你在不实际更改命令的实现的情况下更改命令行为。
        这些属性将与传入 :class:`.Command` 构造函数的属性相同。
    """

    MENTION_TRANSFORMS = {
        '@everyone': '@\u200beveryone',
        r'<@!?[0-9]{17,22}>': '@deleted-user',
        r'<@&[0-9]{17,22}>': '@deleted-role',
    }

    MENTION_PATTERN = re.compile('|'.join(MENTION_TRANSFORMS.keys()))

    def __new__(cls, *args, **kwargs):
        # To prevent race conditions of a single instance while also allowing
        # for settings to be passed the original arguments passed must be assigned
        # to allow for easier copies (which will be made when the help command is actually called)
        # see issue 2123
        self = super().__new__(cls)

        # Shallow copies cannot be used in this case since it is not unusual to pass
        # instances that need state, e.g. Paginator or what have you into the function
        # The keys can be safely copied as-is since they're 99.99% certain of being
        # string keys
        deepcopy = copy.deepcopy
        self.__original_kwargs__ = {k: deepcopy(v) for k, v in kwargs.items()}
        self.__original_args__ = deepcopy(args)
        return self

    def __init__(self, **options):
        self.show_hidden = options.pop('show_hidden', False)
        self.verify_checks = options.pop('verify_checks', True)
        self.command_attrs = attrs = options.pop('command_attrs', {})
        attrs.setdefault('name', 'help')
        attrs.setdefault('help', '显示此消息')
        self.context: Context = qq.utils.MISSING
        self._command_impl = _HelpCommandImpl(self, **self.command_attrs)

    def copy(self):
        obj = self.__class__(*self.__original_args__, **self.__original_kwargs__)
        obj._command_impl = self._command_impl
        return obj

    def _add_to_bot(self, bot):
        command = _HelpCommandImpl(self, **self.command_attrs)
        bot.add_command(command)
        self._command_impl = command

    def _remove_from_bot(self, bot):
        bot.remove_command(self._command_impl.name)
        self._command_impl._eject_cog()

    def add_check(self, func):
        """向帮助命令添加检查。

        Parameters
        ----------
        func
            将用作检查的函数。
        """

        self._command_impl.add_check(func)

    def remove_check(self, func):
        """
        从帮助命令中删除检查。

        此函数是幂等的，如果该函数不在命令的检查中，则不会引发异常。

        Parameters
        ----------
        func
            要从检查中删除的函数。
        """

        self._command_impl.remove_check(func)

    def get_bot_mapping(self):
        """检索传递给 :meth:`send_bot_help` 的机器人映射。"""
        bot = self.context.bot
        mapping = {cog: cog.get_commands() for cog in bot.cogs.values()}
        mapping[None] = [c for c in bot.commands if c.cog is None]
        return mapping

    @property
    def invoked_with(self):
        """与 :attr:`Context.invoked_with` 类似，但是正确处理使用 :meth:`Context.send_help` 的情况。

        如果经常使用帮助命令，则返回 :attr:`Context.invoked_with` 属性。
        否则，如果使用 :meth:`Context.send_help` 调用了帮助命令，则它返回帮助命令的内部命令名称。

        Returns
        ---------
        :class:`str`
            触发此调用的命令名称。
        """
        command_name = self._command_impl.name
        ctx = self.context
        if ctx is None or ctx.command is None or ctx.command.qualified_name != command_name:
            return command_name
        return ctx.invoked_with

    def get_command_signature(self, command):
        """检索帮助页面的签名部分。

        Parameters
        ------------
        command: :class:`Command`
            获取签名的命令。

        Returns
        --------
        :class:`str`
            命令的签名。
        """

        parent = command.parent
        entries = []
        while parent is not None:
            if not parent.signature or parent.invoke_without_command:
                entries.append(parent.name)
            else:
                entries.append(parent.name + ' ' + parent.signature)
            parent = parent.parent
        parent_sig = ' '.join(reversed(entries))

        if len(command.aliases) > 0:
            aliases = '|'.join(command.aliases)
            fmt = f'[{command.name}|{aliases}]'
            if parent_sig:
                fmt = parent_sig + ' ' + fmt
            alias = fmt
        else:
            alias = command.name if not parent_sig else parent_sig + ' ' + command.name

        return f'{self.context.clean_prefix}{alias} {command.signature}'

    def remove_mentions(self, string):
        """从字符串中删除提及以防止滥用。

        这包括 “@所有人”、成员提及和身份组提及。

        Returns
        -------
        :class:`str`
            删除了提及的字符串。
        """

        def replace(obj, *, transforms=self.MENTION_TRANSFORMS):
            return transforms.get(obj.group(0), '@invalid')

        return self.MENTION_PATTERN.sub(replace, string)

    @property
    def cog(self):
        """用于检索或设置帮助命令的齿轮的属性。

        当为 help 命令设置一个齿轮时，就好像 help 命令属于该齿轮。所有齿轮特殊方法都将应用于 help 命令，并且会在卸载时自动取消设置。

        要从帮助命令中取消绑定齿轮，你可以将其设置为 ``None``。

        Returns
        --------
        Optional[:class:`Cog`]
            当前为 help 命令设置的齿轮。
        """
        return self._command_impl.cog

    @cog.setter
    def cog(self, cog):
        # Remove whatever cog is currently valid, if any
        self._command_impl._eject_cog()

        # If a new cog is set then inject it.
        if cog is not None:
            self._command_impl._inject_into_cog(cog)

    def command_not_found(self, string):
        """|maybecoro|

        在帮助命令中找不到命令时调用的方法。这对于 i18n 很有用。

        默认为 ``未找到名为 {0} 的命令。``

        Parameters
        ------------
        string: :class:`str`
            包含无效命令的字符串。请注意，这已被删除以防止滥用。

        Returns
        ---------
        :class:`str`
            未找到命令时使用的字符串。
        """
        return f'未找到名为“{string}”的命令。'

    def subcommand_not_found(self, command, string):
        """|maybecoro|

        当命令没有在帮助命令中请求的子命令时调用的方法。这对于 i18n 很有用。

        Defaults to either:

        - ``'命令“{command.qualified_name}”没有子命令。'``
            - 如果 ``command`` 参数中没有子命令。
        - ``'命令“{command.qualified_name}”没有名为 {string} 的子命令'``
            - 如果 ``command`` 参数有子命令，但没有名为 ``string`` 的子命令。

        Parameters
        ------------
        command: :class:`Command`
            没有请求子命令的命令。
        string: :class:`str`
            包含无效子命令的字符串。请注意，这已被删除以防止滥用。

        Returns
        ---------
        :class:`str`
            当命令没有请求子命令时使用的字符串。
        """
        if isinstance(command, Group) and len(command.all_commands) > 0:
            return f'命令“{command.qualified_name}”没有名为 {string} 的子命令'
        return f'命令“{command.qualified_name}”没有子命令。'

    async def filter_commands(self, commands, *, sort=False, key=None):
        """|coro|

        返回过滤后的命令列表并可选择对它们进行排序。

        这考虑了 :attr:`verify_checks` 和 :attr:`show_hidden` 属性。

        Parameters
        ------------
        commands: Iterable[:class:`Command`]
            被过滤的命令的迭代。
        sort: :class:`bool`
            是否对结果进行排序。
        key: Optional[Callable[:class:`Command`, Any]]
            传递给 :func:`py:sorted` 的可选键函数，它以 :class:`Command` 作为其唯一参数。
            如果 ``sort`` 作为 ``True`` 传递，那么这将默认为命令名称。

        Returns
        ---------
        List[:class:`Command`]
            通过过滤器的命令列表。
        """

        if sort and key is None:
            key = lambda c: c.name

        iterator = commands if self.show_hidden else filter(lambda c: not c.hidden, commands)

        if self.verify_checks is False:
            # if we do not need to verify the checks then we can just
            # run it straight through normally without using await.
            return sorted(iterator, key=key) if sort else list(iterator)

        if self.verify_checks is None and not self.context.guild:
            # if verify_checks is None and we're in a DM, don't verify
            return sorted(iterator, key=key) if sort else list(iterator)

        # if we're here then we need to check every command if it can run
        async def predicate(cmd):
            try:
                return await cmd.can_run(self.context)
            except CommandError:
                return False

        ret = []
        for cmd in iterator:
            valid = await predicate(cmd)
            if valid:
                ret.append(cmd)

        if sort:
            ret.sort(key=key)
        return ret

    def get_max_size(self, commands):
        """返回指定命令列表的最大名称长度。

        Parameters
        ------------
        commands: Sequence[:class:`Command`]
            检查最大尺寸的一系列命令。

        Returns
        --------
        :class:`int`
            命令的最大宽度。
        """

        as_lengths = (qq.utils._string_width(c.name) for c in commands)
        return max(as_lengths, default=0)

    def get_destination(self):
        """返回 :class:`~qq.abc.Messageable` 将在其中输出帮助命令。

        你可以覆盖此方法以自定义行为。

        默认情况下，这将返回 context 的子频道。

        Returns
        -------
        :class:`.abc.Messageable`
            将输出帮助命令的目的地。
        """
        return self.context.channel

    async def send_error_message(self, error):
        """|coro|

        在帮助命令中发生错误时处理实现。例如 :meth:`command_not_found` 的结果会被传递到这里。

        你可以覆盖此方法以自定义行为。

        默认情况下，这会将错误消息发送到 :meth:`get_destination` 指定的目的地。

        .. note::

            你可以使用 :attr:`HelpCommand.context` 访问调用 context 。

        Parameters
        ------------
        error: :class:`str`
            要向用户显示的错误消息。请注意，这已被删除以防止滥用。
        """
        destination = self.get_destination()
        await destination.send(error)

    @_not_overriden
    async def on_help_command_error(self, ctx, error):
        """|coro|

        帮助命令的错误处理程序，由 :ref:`ext_commands_error_handler` 指定。

        如果在调用错误处理程序时需要某些特定行为，则可用于覆盖。

        默认情况下，此方法不执行任何操作，只会传播到默认错误处理程序。

        Parameters
        ------------
        ctx: :class:`Context`
            调用 context 。
        error: :class:`CommandError`
            引发的错误。
        """
        pass

    async def send_bot_help(self, mapping):
        """|coro|

        处理帮助命令中 bot 命令页面的实现。当不带参数调用帮助命令时调用此函数。

        应该注意的是，这个方法不返回任何东西——而是应该在这个方法内部完成实际的消息发送。
        行为良好的子类应该使用 :meth:`get_destination` 来知道发送到哪里，因为这是其他用户的自定义点。

        你可以覆盖此方法以自定义行为。

        .. note::

            你可以使用 :attr:`HelpCommand.context` 访问调用 context 。

            此外，映射中的命令不会被过滤。要进行过滤，你必须自己调用 :meth:`filter_commands`。

        Parameters
        ------------
        mapping: Mapping[Optional[:class:`Cog`], List[:class:`Command`]]
            齿轮到用户请求帮助的命令的映射。映射的键是命令所属的 :class:`~.commands.Cog`，
            如果没有，则为 ``None`` ，值是属于该齿轮的命令列表.
        """
        return None

    async def send_cog_help(self, cog):
        """|coro|

        处理帮助命令中齿轮页面的实现。当使用齿轮作为参数调用 ``help`` 命令时，将调用此函数。

        应该注意的是，这个方法不返回任何东西——而是应该在这个方法内部完成实际的消息发送。
        行为良好的子类应该使用 :meth:`get_destination` 来知道发送到哪里，因为这是其他用户的自定义点。

        你可以覆盖此方法以自定义行为。

        .. note::

            你可以使用 :attr:`HelpCommand.context` 访问调用 context 。

            要获取属于该齿轮的命令，请参见  :meth:`Cog.get_commands` 。
            返回的命令未过滤。要进行过滤，你必须自己调用 :meth:`filter_commands`。

        Parameters
        -----------
        cog: :class:`Cog`
            被请求帮助的齿轮。
        """
        return None

    async def send_group_help(self, group):
        """|coro|

        处理帮助命令中组页面的实现。
        当使用组作为参数调用帮助命令时，将调用此函数。

        应该注意的是，这个方法不返回任何东西——而是应该在这个方法内部完成实际的消息发送。
        行为良好的子类应该使用 :meth:`get_destination` 来知道发送到哪里，因为这是其他用户的自定义点。

        你可以覆盖此方法以自定义行为。

        .. note::

            你可以使用 :attr:`HelpCommand.context` 访问调用 context 。

            要获取不带别名的属于该组的命令，请参阅 :attr:`Group.commands`。
            返回的命令未过滤。要进行过滤，你必须自己调用 :meth:`filter_commands`。

        Parameters
        -----------
        group: :class:`Group`
            被请求帮助的组。
        """
        return None

    async def send_command_help(self, command):
        """|coro|

        处理帮助命令中单个命令页面的实现。

        应该注意的是，这个方法不返回任何东西——而是应该在这个方法内部完成实际的消息发送。
        行为良好的子类应该使用 :meth:`get_destination` 来知道发送到哪里，因为这是其他用户的自定义点。

        你可以覆盖此方法以自定义行为。

        .. note::

            你可以使用 :attr:`HelpCommand.context` 访问调用 context 。

        .. admonition:: 显示帮助
            :class: helpful

            某些属性和方法有助于帮助命令显示，例如：

            - :attr:`Command.help`
            - :attr:`Command.brief`
            - :attr:`Command.short_doc`
            - :attr:`Command.description`
            - :meth:`get_command_signature`

            不是只有是这些属性，但你可以从随意使用这些属性开始来帮助你开始获得所需的输出。

        Parameters
        -----------
        command: :class:`Command`
            请求帮助的命令。
        """
        return None

    async def prepare_help_command(self, ctx, command=None):
        """|coro|

        一种低级方法，可用于在执行任何操作之前准备帮助命令。
        例如，如果你需要在命令进行处理之前在子类中准备一些状态，那么这将是执行此操作的地方。

        默认实现什么都不做。

        .. note::

            这在帮助命令回调主体 **内** 调用。因此，内部发生的所有常见规则也适用于此。

        Parameters
        -----------
        ctx: :class:`Context`
            调用 context 。
        command: Optional[:class:`str`]
            传递给 help 命令的参数。
        """
        pass

    async def command_callback(self, ctx, *, command=None):
        """|coro|

        help 命令的实际执行。

        不建议覆盖此方法，而是通过实际调度的方法更改行为。

        - :meth:`send_bot_help`
        - :meth:`send_cog_help`
        - :meth:`send_group_help`
        - :meth:`send_command_help`
        - :meth:`get_destination`
        - :meth:`command_not_found`
        - :meth:`subcommand_not_found`
        - :meth:`send_error_message`
        - :meth:`on_help_command_error`
        - :meth:`prepare_help_command`
        """
        await self.prepare_help_command(ctx, command)
        bot = ctx.bot

        if command is None:
            mapping = self.get_bot_mapping()
            return await self.send_bot_help(mapping)

        # Check if it's a cog
        cog = bot.get_cog(command)
        if cog is not None:
            return await self.send_cog_help(cog)

        maybe_coro = qq.utils.maybe_coroutine

        # If it's not a cog then it's a command.
        # Since we want to have detailed errors when someone
        # passes an invalid subcommand, we need to walk through
        # the command group chain ourselves.
        keys = command.split(' ')
        cmd = bot.all_commands.get(keys[0])
        if cmd is None:
            string = await maybe_coro(self.command_not_found, self.remove_mentions(keys[0]))
            return await self.send_error_message(string)

        for key in keys[1:]:
            try:
                found = cmd.all_commands.get(key)
            except AttributeError:
                string = await maybe_coro(self.subcommand_not_found, cmd, self.remove_mentions(key))
                return await self.send_error_message(string)
            else:
                if found is None:
                    string = await maybe_coro(self.subcommand_not_found, cmd, self.remove_mentions(key))
                    return await self.send_error_message(string)
                cmd = found

        if isinstance(cmd, Group):
            return await self.send_group_help(cmd)
        else:
            return await self.send_command_help(cmd)


class DefaultHelpCommand(HelpCommand):
    """默认帮助命令的执行。

    这继承自:class:`HelpCommand`。

    它使用以下属性对其进行了扩展。

    Attributes
    ------------
    width: :class:`int`
        适合一行的最大字符数。默认为 80。
    sort_commands: :class:`bool`
        是否按字母顺序对输出中的命令进行排序。默认为“真”。
    dm_help: Optional[:class:`bool`]
        一个三个选择参数，指示帮助命令是否应该私聊用户而不是将其发送到它接收它的频道。
        如果布尔值设置为 ``True`` ，则所有帮助输出都是私聊的。
        如果 ``False`` ，则没有任何帮助输出是私聊。
        如果 ``None`` ，那么当帮助消息变得太长（由超过 :attr:`dm_help_threshold` 字符决定）时，机器人只会发送私聊。
        默认为 ``False`` 。
    dm_help_threshold: Optional[:class:`int`]
        如果 :attr:`dm_help` 设置为 ``None`` ，则分页器在将私聊发送给用户之前必须累积的字符数。默认为 ``1000`` 。
    indent: :class:`int`
        从标题中缩进命令的程度。默认为 ``2`` 。
    commands_heading: :class:`str`
        当使用类别名称调用帮助命令时使用的命令列表的标题字符串。对 i18n 有用。默认为 ``"命令："``
    no_category: :class:`str`
        存在不属于任何类别（cog）的命令时使用的字符串。对 i18n 有用。默认为 ``"无类别" ``
    paginator: :class:`Paginator`
        用于对帮助命令输出进行分页的分页器。
    """

    def __init__(self, **options):
        self.width = options.pop('width', 80)
        self.indent = options.pop('indent', 2)
        self.sort_commands = options.pop('sort_commands', True)
        self.dm_help = options.pop('dm_help', False)
        self.dm_help_threshold = options.pop('dm_help_threshold', 1000)
        self.commands_heading = options.pop('commands_heading', "命令：")
        self.no_category = options.pop('no_category', '无类别')
        self.paginator = options.pop('paginator', None)

        if self.paginator is None:
            self.paginator = Paginator()

        super().__init__(**options)

    def shorten_text(self, text):
        """:class:`str`: 缩短文本以适应 :attr:`width`。"""
        if len(text) > self.width:
            return text[:self.width - 3].rstrip() + '...'
        return text

    def get_ending_note(self):
        """:class:`str`: 返回帮助命令的结束注释。这主要用于覆盖 i18n 目的。"""
        command_name = self.invoked_with
        return (
            f"输入 {self.context.clean_prefix}{command_name} 命令以获取有关命令的更多信息。\n"
            f"你还可以输入 {self.context.clean_prefix}{command_name} 类别 以获取有关类别的更多信息。"
        )

    def add_indented_commands(self, commands, *, heading, max_size=None):
        """在指定标题后缩进命令列表。

        格式添加到 :attr:`paginator`。

        默认实现是命令名称由 :attr:`indent` 空格缩进，
        填充到 ``max_size`` 后跟命令的 :attr:`Command.short_doc` 然后缩短以适应 :attr:`width`。

        Parameters
        -----------
        commands: Sequence[:class:`Command`]
            用于缩进输出的命令列表。
        heading: :class:`str`
            要添加到输出的标题。仅当命令列表大于 0 时才添加此项。
        max_size: Optional[:class:`int`]
            用于缩进间隙的最大尺寸。如果未指定，则在命令参数上调用 :meth:`~HelpCommand.get_max_size`。
        """

        if not commands:
            return

        self.paginator.add_line(heading)
        max_size = max_size or self.get_max_size(commands)

        get_width = qq.utils._string_width
        for command in commands:
            name = command.name
            width = max_size - (get_width(name) - len(name))
            entry = f'{self.indent * " "}{name:<{width}} {command.short_doc}'
            self.paginator.add_line(self.shorten_text(entry))

    async def send_pages(self):
        """将页面输出从 :attr:`paginator` 发送到目的地的辅助实用程序。"""
        destination = self.get_destination()
        for page in self.paginator.pages:
            await destination.send(page)

    def add_command_formatting(self, command):
        """用于格式化命令和组的非缩进块的实用函数。

        Parameters
        ------------
        command: :class:`Command`
            要格式化的命令。
        """

        if command.description:
            self.paginator.add_line(command.description, empty=True)

        signature = self.get_command_signature(command)
        self.paginator.add_line(signature, empty=True)

        if command.help:
            try:
                self.paginator.add_line(command.help, empty=True)
            except RuntimeError:
                for line in command.help.splitlines():
                    self.paginator.add_line(line)
                self.paginator.add_line()

    def get_destination(self):
        ctx = self.context
        if self.dm_help is True:
            return ctx.author
        elif self.dm_help is None and len(self.paginator) > self.dm_help_threshold:
            return ctx.author
        else:
            return ctx.channel

    async def prepare_help_command(self, ctx, command):
        self.paginator.clear()
        await super().prepare_help_command(ctx, command)

    async def send_bot_help(self, mapping):
        ctx = self.context
        bot = ctx.bot

        if bot.description:
            # <description> portion
            self.paginator.add_line(bot.description, empty=True)

        no_category = f'\u200b{self.no_category}:'

        def get_category(command, *, no_category=no_category):
            cog = command.cog
            return cog.qualified_name + ':' if cog is not None else no_category

        filtered = await self.filter_commands(bot.commands, sort=True, key=get_category)
        max_size = self.get_max_size(filtered)
        to_iterate = itertools.groupby(filtered, key=get_category)

        # Now we can add the commands to the page.
        for category, commands in to_iterate:
            commands = sorted(commands, key=lambda c: c.name) if self.sort_commands else list(commands)
            self.add_indented_commands(commands, heading=category, max_size=max_size)

        note = self.get_ending_note()
        if note:
            self.paginator.add_line()
            self.paginator.add_line(note)

        await self.send_pages()

    async def send_command_help(self, command):
        self.add_command_formatting(command)
        self.paginator.close_page()
        await self.send_pages()

    async def send_group_help(self, group):
        self.add_command_formatting(group)

        filtered = await self.filter_commands(group.commands, sort=self.sort_commands)
        self.add_indented_commands(filtered, heading=self.commands_heading)

        if filtered:
            note = self.get_ending_note()
            if note:
                self.paginator.add_line()
                self.paginator.add_line(note)

        await self.send_pages()

    async def send_cog_help(self, cog):
        if cog.description:
            self.paginator.add_line(cog.description, empty=True)

        filtered = await self.filter_commands(cog.get_commands(), sort=self.sort_commands)
        self.add_indented_commands(filtered, heading=self.commands_heading)

        note = self.get_ending_note()
        if note:
            self.paginator.add_line()
            self.paginator.add_line(note)

        await self.send_pages()


class MinimalHelpCommand(HelpCommand):
    """具有最少输出的帮助命令的实现。

    这继承自 :class:`HelpCommand`。

    Attributes
    ------------
    sort_commands: :class:`bool`
        是否按字母顺序对输出中的命令进行排序。默认为 ``True`` 。
    commands_heading: :class:`str`
        当使用类别名称调用帮助命令时使用的命令列表的标题字符串。对 i18n 有用。默认为 ``“命令”`` 。
    aliases_heading: :class:`str`
        别名列表的标题字符串用于列出命令的别名。对 i18n 有用。默认为``"别名："`` 。
    dm_help: Optional[:class:`bool`]
        一个三个选择参数，指示帮助命令是否应该私聊用户而不是将其发送到它接收它的频道。
        如果布尔值设置为 ``True`` ，则所有帮助输出都是私聊的。
        如果 ``False`` ，则没有任何帮助输出是私聊。
        如果 ``None`` ，那么当帮助消息变得太长（由超过 :attr:`dm_help_threshold` 字符决定）时，机器人只会发送私聊。
        默认为 ``False`` 。
    dm_help_threshold: Optional[:class:`int`]
        如果 :attr:`dm_help` 设置为 ``None`` ，则分页器在将私聊发送给用户之前必须累积的字符数。默认为 ``1000`` 。
    no_category: :class:`str`
        存在不属于任何类别（cog）的命令时使用的字符串。对 i18n 有用。默认为 ``"无类别:" ``
    paginator: :class:`Paginator`
        用于对帮助命令输出进行分页的分页器。
    """

    def __init__(self, **options):
        self.sort_commands = options.pop('sort_commands', True)
        self.commands_heading = options.pop('commands_heading', "命令")
        self.dm_help = options.pop('dm_help', False)
        self.dm_help_threshold = options.pop('dm_help_threshold', 1000)
        self.aliases_heading = options.pop('aliases_heading', "别名：")
        self.no_category = options.pop('no_category', '无类别')
        self.paginator = options.pop('paginator', None)

        if self.paginator is None:
            self.paginator = Paginator(suffix=None, prefix=None)

        super().__init__(**options)

    async def send_pages(self):
        """将页面输出从 :attr:`paginator` 发送到目的地的辅助实用程序。"""
        destination = self.get_destination()
        for page in self.paginator.pages:
            await destination.send(page)

    def get_opening_note(self):
        """返回帮助命令的开头注释。这主要用于覆盖 i18n 目的。

        默认实现返回 ::

            使用 `{prefix}{command_name} [命令]` 获取有关命令的更多信息。
            你还可以使用 `{prefix}{command_name} [类别]` 获取有关类别的更多信息。

        Returns
        -------
        :class:`str`
            The help command opening note.
        """
        command_name = self.invoked_with
        return (
            f"使用 `{self.context.clean_prefix}{command_name} [命令]` 获取有关命令的更多信息。\n"
            f"你还可以使用 `{self.context.clean_prefix}{command_name} [类别]` 获取有关类别的更多信息。"
        )

    def get_command_signature(self, command):
        return f'{self.context.clean_prefix}{command.qualified_name} {command.signature}'

    def get_ending_note(self):
        """返回帮助命令的结束注释。这主要用于覆盖 i18n 目的。

        默认实现什么都不做。

        Returns
        -------
        :class:`str`
            帮助命令结束注释。
        """
        return None

    def add_bot_commands_formatting(self, commands, heading):
        """将带有命令的缩小机器人标题添加到输出中。

        格式应该添加到 :attr:`paginator`。

        默认实现是一个标题，下一行是由 EN SPACE (U+2002) 分隔的命令。

        Parameters
        -----------
        commands: Sequence[:class:`Command`]
            属于标题的命令列表。
        heading: :class:`str`
            要添加到行的标题。
        """
        if commands:
            # U+2002 Middle Dot
            joined = '\u2002'.join(c.name for c in commands)
            self.paginator.add_line(f'__**{heading}**__')
            self.paginator.add_line(joined)

    def add_subcommand_formatting(self, command):
        """在子命令上添加格式信息。

        格式应该添加到 :attr:`paginator`。

        默认实现是前缀和:attr:`Command.qualified_name` 可选后跟一个短划线和命令的:attr:`Command.short_doc`。

        Parameters
        -----------
        command: :class:`Command`
            显示信息的命令。
        """
        fmt = '{0}{1} \N{EN DASH} {2}' if command.short_doc else '{0}{1}'
        self.paginator.add_line(fmt.format(self.context.clean_prefix, command.qualified_name, command.short_doc))

    def add_aliases_formatting(self, aliases):
        """添加有关命令别名的格式信息。

        格式应该添加到 :attr:`paginator`。

        默认实现是 :attr:`aliases_heading` 跟逗号分隔的别名列表。

        如果没有要格式化的别名，则不会调用此方法。

        Parameters
        -----------
        aliases: Sequence[:class:`str`]
            要格式化的别名列表。
        """
        self.paginator.add_line(f'**{self.aliases_heading}** {", ".join(aliases)}', empty=True)

    def add_command_formatting(self, command):
        """用于格式化命令和组的实用程序函数。

        Parameters
        ------------
        command: :class:`Command`
            要格式化的命令。
        """

        if command.description:
            self.paginator.add_line(command.description, empty=True)

        signature = self.get_command_signature(command)
        if command.aliases:
            self.paginator.add_line(signature)
            self.add_aliases_formatting(command.aliases)
        else:
            self.paginator.add_line(signature, empty=True)

        if command.help:
            try:
                self.paginator.add_line(command.help, empty=True)
            except RuntimeError:
                for line in command.help.splitlines():
                    self.paginator.add_line(line)
                self.paginator.add_line()

    def get_destination(self):
        ctx = self.context
        if self.dm_help is True:
            return ctx.author
        elif self.dm_help is None and len(self.paginator) > self.dm_help_threshold:
            return ctx.author
        else:
            return ctx.channel

    async def prepare_help_command(self, ctx, command):
        self.paginator.clear()
        await super().prepare_help_command(ctx, command)

    async def send_bot_help(self, mapping):
        ctx = self.context
        bot = ctx.bot

        if bot.description:
            self.paginator.add_line(bot.description, empty=True)

        note = self.get_opening_note()
        if note:
            self.paginator.add_line(note, empty=True)

        no_category = f'\u200b{self.no_category}'

        def get_category(command, *, no_category=no_category):
            cog = command.cog
            return cog.qualified_name if cog is not None else no_category

        filtered = await self.filter_commands(bot.commands, sort=True, key=get_category)
        to_iterate = itertools.groupby(filtered, key=get_category)

        for category, commands in to_iterate:
            commands = sorted(commands, key=lambda c: c.name) if self.sort_commands else list(commands)
            self.add_bot_commands_formatting(commands, category)

        note = self.get_ending_note()
        if note:
            self.paginator.add_line()
            self.paginator.add_line(note)

        await self.send_pages()

    async def send_cog_help(self, cog):
        bot = self.context.bot
        if bot.description:
            self.paginator.add_line(bot.description, empty=True)

        note = self.get_opening_note()
        if note:
            self.paginator.add_line(note, empty=True)

        if cog.description:
            self.paginator.add_line(cog.description, empty=True)

        filtered = await self.filter_commands(cog.get_commands(), sort=self.sort_commands)
        if filtered:
            self.paginator.add_line(f'**{cog.qualified_name} {self.commands_heading}**')
            for command in filtered:
                self.add_subcommand_formatting(command)

            note = self.get_ending_note()
            if note:
                self.paginator.add_line()
                self.paginator.add_line(note)

        await self.send_pages()

    async def send_group_help(self, group):
        self.add_command_formatting(group)

        filtered = await self.filter_commands(group.commands, sort=self.sort_commands)
        if filtered:
            note = self.get_opening_note()
            if note:
                self.paginator.add_line(note, empty=True)

            self.paginator.add_line(f'**{self.commands_heading}**')
            for command in filtered:
                self.add_subcommand_formatting(command)

            note = self.get_ending_note()
            if note:
                self.paginator.add_line()
                self.paginator.add_line(note)

        await self.send_pages()

    async def send_command_help(self, command):
        self.add_command_formatting(command)
        self.paginator.close_page()
        await self.send_pages()



from typing import List, TypedDict, Optional


class EmbedField(TypedDict, total=False):
    name: str
    value: str


class Embed(TypedDict, total=False):
    title: str
    description: str
    prompt: str
    timestamp: str
    fields: List[EmbedField]


class ArkObjKv(TypedDict, total=False):
    key: str
    value: str


class ArkObj(TypedDict, total=False):
    obj_kv: List[ArkObjKv]


class ArkKv(TypedDict, total=False):
    key: str
    value: Optional[str]
    obj: Optional[List[ArkObj]]


class Ark(TypedDict, total=False):
    template_id: int
    kv: List[ArkKv]



from __future__ import annotations

import base64
from typing import Optional, TYPE_CHECKING, Union

import os
import io

__all__ = (
    'File',
)


class File:
    r"""用于 :meth:`abc.Messageable.send` 的参数对象，用于发送文件对象。

    .. note::

        文件对象是一次性的，不能在多个 :meth:`abc.Messageable.send` 中重复使用。

    .. warning::

        本功能尚未被官方实现，实现为 Discord 的实现

    Attributes
    -----------
    fp: Union[:class:`os.PathLike`, :class:`io.BufferedIOBase`]
        以二进制模式和读取模式打开的类文件对象或表示硬盘中要打开的文件的文件名。

        .. note::

            如果传递的类文件对象是通过 ``open`` 打开的，则应使用 ``rb`` 模式。 要传递二进制数据，请考虑使用 ``io.BytesIO``。

    filename: Optional[:class:`str`]
        上传到 QQ 时显示的文件名。
        如果没有给出，那么它默认为 ``fp.name`` 或者如果 ``fp`` 是一个字符串，那么 ``filename`` 将默认为给定的字符串。
    """

    __slots__ = ('fp', 'filename', '_original_pos', '_owner', '_closer')

    if TYPE_CHECKING:
        fp: io.BufferedIOBase
        filename: Optional[str]

    def __init__(
        self,
        fp: Union[str, bytes, os.PathLike, io.BufferedIOBase],
        filename: Optional[str] = None,
    ):
        if isinstance(fp, io.IOBase):
            if not (fp.seekable() and fp.readable()):
                raise ValueError(f'File buffer {fp!r} must be seekable and readable')
            self.fp = fp
            self._original_pos = fp.tell()
            self._owner = False
        else:
            self.fp = open(fp, 'rb')
            self._original_pos = 0
            self._owner = True

        # aiohttp only uses two methods from IOBase
        # read and close, since I want to control when the files
        # close, I need to stub it so it doesn't close unless
        # I tell it to
        self._closer = self.fp.close
        self.fp.close = lambda: None

        if filename is None:
            if isinstance(fp, str):
                _, self.filename = os.path.split(fp)
            else:
                self.filename = getattr(fp, 'name', None)
        else:
            self.filename = filename

    def reset(self, *, seek: Union[int, bool] = True) -> None:
        if seek:
            self.fp.seek(self._original_pos)

    def close(self) -> None:
        self.fp.close = self._closer
        if self._owner:
            self._closer()



from __future__ import annotations

from typing import Optional, Any, TYPE_CHECKING, List, Callable, Type, Tuple, Union

from qq.error import ClientException, QQException
from ... import Role

if TYPE_CHECKING:
    from inspect import Parameter

    from .converter import Converter
    from .context import Context
    from .cooldowns import Cooldown, BucketType
    from .flags import Flag
    from qq.abc import GuildChannel

__all__ = (
    'CommandError',
    'MissingRequiredArgument',
    'BadArgument',
    'PrivateMessageOnly',
    'NoPrivateMessage',
    'CheckFailure',
    'CheckAnyFailure',
    'CommandNotFound',
    'DisabledCommand',
    'CommandInvokeError',
    'TooManyArguments',
    'UserInputError',
    'CommandOnCooldown',
    'MaxConcurrencyReached',
    'NotOwner',
    'MessageNotFound',
    'ObjectNotFound',
    'MemberNotFound',
    'GuildNotFound',
    'UserNotFound',
    'ChannelNotFound',
    'ChannelNotReadable',
    'BadColourArgument',
    'BadColorArgument',
    'RoleNotFound',
    'BadBoolArgument',
    'MissingRole',
    'BotMissingRole',
    'MissingAnyRole',
    'BotMissingAnyRole',
    'MissingPermissions',
    'BotMissingPermissions',
    'ConversionError',
    'BadUnionArgument',
    'BadLiteralArgument',
    'ArgumentParsingError',
    'UnexpectedQuoteError',
    'InvalidEndOfQuotedStringError',
    'ExpectedClosingQuoteError',
    'ExtensionError',
    'ExtensionAlreadyLoaded',
    'ExtensionNotLoaded',
    'NoEntryPointError',
    'ExtensionFailed',
    'ExtensionNotFound',
    'CommandRegistrationError',
    'FlagError',
    'BadFlagArgument',
    'MissingFlagArgument',
    'TooManyFlags',
    'MissingRequiredFlag',
)


class CommandError(QQException):
    r"""所有命令相关错误的基本异常类型。

    这继承自 :exc:`qq.QQException`。

    这个异常和从它继承的异常以一种特殊的方式被处理，因为它们被捕获并从 :class:`.Bot`\, :func:`.on_command_error` 传递到一个特殊的事件中。
    """

    def __init__(self, message: Optional[str] = None, *args: Any) -> None:
        if message is not None:
            # clean-up @everyone and @here mentions
            m = message.replace('@everyone', '@\u200beveryone').replace('@here', '@\u200bhere')
            super().__init__(m, *args)
        else:
            super().__init__(*args)


class ConversionError(CommandError):
    """当 Converter 类引发非 CommandError 时引发异常。

    这继承自 :exc:`CommandError` 。

    Attributes
    ----------
    converter: :class:`qq.ext.commands.Converter`
        失败的转换器。
    original: :exc:`Exception`
        引发的原始异常。 你也可以通过 ``__cause__`` 属性获取此信息。
    """

    def __init__(self, converter: Converter, original: Exception) -> None:
        self.converter: Converter = converter
        self.original: Exception = original


class UserInputError(CommandError):
    """涉及用户输入错误的错误的基本异常类型。

    这继承自 :exc:`CommandError`。
    """
    pass


class CommandNotFound(CommandError):
    """尝试调用命令但未找到该名称下的命令时引发异常。

    这不是针对无效子命令引发的，而只是尝试调用的初始主命令。

    这继承自 :exc:`CommandError` 。
    """
    pass


class MissingRequiredArgument(UserInputError):
    """解析命令时引发异常，并且未遇到所需的参数。

    这继承自 :exc:`UserInputError`

    Attributes
    -----------
    param: :class:`inspect.Parameter`
        缺少参数。
    """

    def __init__(self, param: Parameter) -> None:
        self.param: Parameter = param
        super().__init__(f'{param.name} 是缺少的必需参数。')


class TooManyArguments(UserInputError):
    """当命令传递了太多参数并且其 :attr:`.Command.ignore_extra` 属性未设置为 ``True`` 时引发异常。

    这继承自 :exc:`UserInputError`
    """
    pass


class BadArgument(UserInputError):
    """在传递给命令的参数上遇到解析或转换失败时引发异常。

    这继承自 :exc:`UserInputError`
    """
    pass


class CheckFailure(CommandError):
    """当 :attr:`.Command.checks` 中的谓词失败时引发异常。

    这继承自 :exc:`CommandError`
    """
    pass


class CheckAnyFailure(CheckFailure):
    """当 :func:`check_any` 中的所有谓词都失败时引发异常。

    这继承自 :exc:`CheckFailure`。

    Attributes
    ------------
    errors: List[:class:`CheckFailure`]
        执行期间捕获的错误列表。
    checks: List[Callable[[:class:`Context`], :class:`bool`]]
        失败的检查谓词列表。
    """

    def __init__(self, checks: List[CheckFailure], errors: List[Callable[[Context], bool]]) -> None:
        self.checks: List[CheckFailure] = checks
        self.errors: List[Callable[[Context], bool]] = errors
        super().__init__('你无权运行此命令。')


class PrivateMessageOnly(CheckFailure):
    """当操作在私人消息上下文之外不起作用时引发异常。

    这继承自 :exc:`CheckFailure`
    """

    def __init__(self, message: Optional[str] = None) -> None:
        super().__init__(message or '该命令只能用于私信。')


class NoPrivateMessage(CheckFailure):
    """当操作在私人消息上下文中不起作用时引发异常。

    这继承自 :exc:`CheckFailure`
    """

    def __init__(self, message: Optional[str] = None) -> None:
        super().__init__(message or '该命令不能用于私信。')


class NotOwner(CheckFailure):
    """当消息作者不是机器人的所有者时引发异常。

    这继承自 :exc:`CheckFailure`
    """
    pass


class ObjectNotFound(BadArgument):
    """当提供的参数与 ID 或提及的格式不匹配时引发异常。

    这继承自 :exc:`BadArgument`

    Attributes
    -----------
    argument: :class:`str`
        调用者提供的不匹配的参数
    """

    def __init__(self, argument: str) -> None:
        self.argument: str = argument
        super().__init__(f'{argument!r} 不遵循有效的 ID 或提及格式。')


class MemberNotFound(BadArgument):
    """在机器人的缓存中找不到提供的成员时引发异常。

    这继承自 :exc:`BadArgument`

    Attributes
    -----------
    argument: :class:`str`
        未找到调用者提供的成员
    """

    def __init__(self, argument: str) -> None:
        self.argument: str = argument
        super().__init__(f'未找到成员“{argument}”。')


class GuildNotFound(BadArgument):
    """在机器人的缓存中找不到提供的频道时引发异常。

    这继承自 :exc:`BadArgument`

    Attributes
    -----------
    argument: :class:`str`
        未找到的调用者提供的频道未
    """

    def __init__(self, argument: str) -> None:
        self.argument: str = argument
        super().__init__(f'未找到频道“{argument}”。')


class UserNotFound(BadArgument):
    """当在机器人的缓存中找不到提供的用户时引发异常。

    这继承自 :exc:`BadArgument`

    Attributes
    -----------
    argument: :class:`str`
        未找到的调用者提供的用户
    """

    def __init__(self, argument: str) -> None:
        self.argument: str = argument
        super().__init__(f'未找到用户“{argument}”。')


class MessageNotFound(BadArgument):
    """在频道中找不到提供的消息时引发异常。

    这继承自 :exc:`BadArgument`

    Attributes
    -----------
    argument: :class:`str`
        未找到的调用者提供的消息
    """

    def __init__(self, argument: str) -> None:
        self.argument: str = argument
        super().__init__(f'未找到消息“{argument}”。')


class ChannelNotReadable(BadArgument):
    """当机器人无权读取子频道中的消息时引发异常。

    这继承自 :exc:`BadArgument`

    Attributes
    -----------
    argument: Union[:class:`.abc.GuildChannel`, :class:`.Thread`]
        调用者提供的不可读的子频道
    """

    def __init__(self, argument: Union[GuildChannel]) -> None:
        self.argument: Union[GuildChannel] = argument
        super().__init__(f"无法阅读 {argument.mention} 中的消息。")


class ChannelNotFound(BadArgument):
    """当机器人找不到子频道时引发异常。

    这继承自 :exc:`BadArgument`

    Attributes
    -----------
    argument: :class:`str`
        未找到的调用者提供的子频道
    """

    def __init__(self, argument: str) -> None:
        self.argument: str = argument
        super().__init__(f'未找到子频道“{argument}”。')


class BadColourArgument(BadArgument):
    """颜色无效时引发异常。

    这继承自 :exc:`BadArgument`

    Attributes
    -----------
    argument: :class:`str`
        调用者提供的颜色无效
    """

    def __init__(self, argument: str) -> None:
        self.argument: str = argument
        super().__init__(f'颜色“{argument}”无效。')


BadColorArgument = BadColourArgument


class RoleNotFound(BadArgument):
    """当机器人找不到身份组时引发异常。

    这继承自 :exc:`BadArgument`

    Attributes
    -----------
    argument: :class:`str`
        未找到的调用者提供的身份组
    """

    def __init__(self, argument: str) -> None:
        self.argument: str = argument
        super().__init__(f'未找到身份组“{argument}”。')


class BadBoolArgument(BadArgument):
    """当布尔参数不可转换时引发异常。

    这继承自 :exc:`BadArgument`

    Attributes
    -----------
    argument: :class:`str`
        调用者提供的不在预定义列表中的布尔参数
    """

    def __init__(self, argument: str) -> None:
        self.argument: str = argument
        super().__init__(f'{argument} 不是认识的布尔选项')


class DisabledCommand(CommandError):
    """调用正在禁用的命令时引发异常。

    这继承自 :exc:`CommandError`
    """
    pass


class CommandInvokeError(CommandError):
    """当被调用的命令引发异常时的异常。

    这继承自 :exc:`CommandError`

    Attributes
    -----------
    original: :exc:`Exception`
        引发的原始异常。 你也可以通过 ``__cause__`` 属性获取此信息。
    """

    def __init__(self, e: Exception) -> None:
        self.original: Exception = e
        super().__init__(f'Command raised an exception: {e.__class__.__name__}: {e}')


class CommandOnCooldown(CommandError):
    """当被调用的命令处于冷却状态时引发异常。

    这继承自 :exc:`CommandError`

    Attributes
    -----------
    cooldown: :class:`.Cooldown`
        一个具有 ``rate`` 和 ``per`` 属性的类，类似于 :func:`.cooldown` 装饰器。
    type: :class:`BucketType`
        与冷却时间关联的类型。
    retry_after: :class:`float`
        在你可以重试之前等待的秒数。
    """

    def __init__(self, cooldown: Cooldown, retry_after: float, type: BucketType) -> None:
        self.cooldown: Cooldown = cooldown
        self.retry_after: float = retry_after
        self.type: BucketType = type
        super().__init__(f'你正在冷却中。 在 {retry_after:.2f}s 后重试')


class MaxConcurrencyReached(CommandError):
    """当被调用的命令达到其最大并发时引发异常。

    Attributes
    ------------
    number: :class:`int`
        允许的最大并发调用者数。
    per: :class:`.BucketType`
        传递给 :func:`.max_concurrency` 装饰器的桶类型。
    """

    def __init__(self, number: int, per: BucketType) -> None:
        self.number: int = number
        self.per: BucketType = per
        name = per.name
        suffix = '每 %s' % name if per.name != 'default' else 'globally'
        plural = '%s %s 次'
        fmt = plural % (suffix, number)
        super().__init__(f'太多人在使用这个命令。 它只能在 {fmt} 同时使用。')


class MissingRole(CheckFailure):
    """当命令调用者缺少运行命令的身份组时引发异常。

    这继承自 :exc:`CheckFailure`

    .. versionadded:: 1.1

    Attributes
    -----------
    missing_role: Union[:class:`str`, :class:`int`]
        缺少的必需身份组。
        这是传递给 :func:`~.commands.has_role` 的参数。
    """

    def __init__(self, missing_role: Role) -> None:
        self.missing_role: Role = missing_role
        message = f'运行此命令需要身份组 {missing_role!r}。'
        super().__init__(message)


class BotMissingRole(CheckFailure):
    """当机器人的成员缺乏运行命令的身份组时引发异常。

    这继承自 :exc:`CheckFailure`

    Attributes
    -----------
    missing_role: Union[:class:`str`, :class:`int`]
        缺少的必需身份组。
        这是传递给 :func:`~.commands.has_role` 的参数。
    """

    def __init__(self, missing_role: Role) -> None:
        self.missing_role: Role = missing_role
        message = f'Bot 需要身份组 {missing_role!r} 才能运行此命令'
        super().__init__(message)


class MissingAnyRole(CheckFailure):
    """当命令调用者缺少指定用于运行命令的任何身份组时引发异常。

    这继承自 :exc:`CheckFailure`

    Attributes
    -----------
    missing_roles: List[Union[:class:`str`, :class:`int`]]
        调用者缺少的身份组。
        这些是传递给 :func:`~.commands.has_any_role` 的参数。
    """

    def __init__(self, missing_roles: Role) -> None:
        self.missing_roles: Role = missing_roles

        missing = [f"'{role}'" for role in missing_roles]

        if len(missing) > 2:
            fmt = '{}, or {}'.format(", ".join(missing[:-1]), missing[-1])
        else:
            fmt = ' or '.join(missing)

        message = f"你至少缺少一个必需的身份组：{fmt}"
        super().__init__(message)


class BotMissingAnyRole(CheckFailure):
    """当机器人的成员缺少指定运行命令的任何身份组时引发异常。

    这继承自 :exc:`CheckFailure`

    Attributes
    -----------
    missing_roles: List[Union[:class:`str`, :class:`int`]]
        缺少机器人成员的身份组。
        这些是传递给 :func:`~.commands.has_any_role` 的参数。

    """

    def __init__(self, missing_roles: List[Role]) -> None:
        self.missing_roles: List[Role] = missing_roles

        missing = [f"'{role}'" for role in missing_roles]

        if len(missing) > 2:
            fmt = '{}, or {}'.format(", ".join(missing[:-1]), missing[-1])
        else:
            fmt = ' or '.join(missing)

        message = f"Bot 缺少至少一个必需的身份组：{fmt}"
        super().__init__(message)


class MissingPermissions(CheckFailure):
    """当命令调用者缺乏运行命令的权限时引发异常。

    这继承自 :exc:`CheckFailure`

    Attributes
    -----------
    missing_permissions: List[:class:`str`]
        缺少所需的权限。
    """

    def __init__(self, missing_permissions: List[str], *args: Any) -> None:
        self.missing_permissions: List[str] = missing_permissions

        missing = [perm.replace('_', ' ').replace('guild', 'server').title() for perm in missing_permissions]

        if len(missing) > 2:
            fmt = '{}, and {}'.format(", ".join(missing[:-1]), missing[-1])
        else:
            fmt = ' and '.join(missing)
        message = f'你缺少运行此命令的 {fmt} 权限。'
        super().__init__(message, *args)


class BotMissingPermissions(CheckFailure):
    """当机器人的成员缺乏运行命令的权限时引发异常。

    这继承自 :exc:`CheckFailure`

    Attributes
    -----------
    missing_permissions: List[:class:`str`]
        缺少所需的权限。
    """

    def __init__(self, missing_permissions: List[str], *args: Any) -> None:
        self.missing_permissions: List[str] = missing_permissions

        missing = [perm.replace('_', ' ').replace('guild', 'server').title() for perm in missing_permissions]

        if len(missing) > 2:
            fmt = '{}, and {}'.format(", ".join(missing[:-1]), missing[-1])
        else:
            fmt = ' and '.join(missing)
        message = f'Bot 需要 {fmt} 权限才能运行此命令。'
        super().__init__(message, *args)


class BadUnionArgument(UserInputError):
    """当 :data:`typing.Union` 转换器对其所有关联类型失败时引发异常。

    这继承自 :exc:`UserInputError`

    Attributes
    -----------
    param: :class:`inspect.Parameter`
        转换失败的参数。
    converters: Tuple[Type, ``...``]
        按失败顺序尝试转换的转换器元组。
    errors: List[:class:`CommandError`]
        由于转换失败而捕获的错误列表。
    """

    def __init__(self, param: Parameter, converters: Tuple[Type, ...], errors: List[CommandError]) -> None:
        self.param: Parameter = param
        self.converters: Tuple[Type, ...] = converters
        self.errors: List[CommandError] = errors

        def _get_name(x):
            try:
                return x.__name__
            except AttributeError:
                if hasattr(x, '__origin__'):
                    return repr(x)
                return x.__class__.__name__

        to_string = [_get_name(x) for x in converters]
        if len(to_string) > 2:
            fmt = '{}, or {}'.format(', '.join(to_string[:-1]), to_string[-1])
        else:
            fmt = ' or '.join(to_string)

        super().__init__(f'无法将“{param.name}”转换为 {fmt}。')


class BadLiteralArgument(UserInputError):
    """当 :data:`typing.Literal` 转换器的所有关联值都失败时引发异常。

    这继承自 :exc:`UserInputError`

    Attributes
    -----------
    param: :class:`inspect.Parameter`
        转换失败的参数。
    literals: Tuple[Any, ``...``]
        按失败顺序在转换中比较的一组值。
    errors: List[:class:`CommandError`]
        由于转换失败而捕获的错误列表。
    """

    def __init__(self, param: Parameter, literals: Tuple[Any, ...], errors: List[CommandError]) -> None:
        self.param: Parameter = param
        self.literals: Tuple[Any, ...] = literals
        self.errors: List[CommandError] = errors

        to_string = [repr(l) for l in literals]
        if len(to_string) > 2:
            fmt = '{}, or {}'.format(', '.join(to_string[:-1]), to_string[-1])
        else:
            fmt = ' or '.join(to_string)

        super().__init__(f'无法将“{param.name}”转换为文字 {fmt}。')


class ArgumentParsingError(UserInputError):
    """当解析器无法解析用户的输入时引发异常。

    这继承自 :exc:`UserInputError`。

    有一些子类为 i18n 目的实现更细度的解析错误。
    """
    pass


class UnexpectedQuoteError(ArgumentParsingError):
    """当解析器在非引用字符串中遇到引号时引发异常。

    这继承自 :exc:`ArgumentParsingError`。

    Attributes
    ------------
    quote: :class:`str`
        在非引号字符串中找到的引号。
    """

    def __init__(self, quote: str) -> None:
        self.quote: str = quote
        super().__init__(f'非引号字符串中的意外引号 {quote!r}')


class InvalidEndOfQuotedStringError(ArgumentParsingError):
    """当字符串中的结束引号后需要空格但发现不同的字符时引发异常。

    这继承自 :exc:`ArgumentParsingError`。

    Attributes
    -----------
    char: :class:`str`
        找到的字符而不是预期的字符串。
    """

    def __init__(self, char: str) -> None:
        self.char: str = char
        super().__init__(f'结束引号后的预期空格但收到 {char!r}')


class ExpectedClosingQuoteError(ArgumentParsingError):
    """当需要引号字符但未找到时引发异常。

    这继承自 :exc:`ArgumentParsingError`。

    Attributes
    -----------
    close_quote: :class:`str`
        预期的引号字符。
    """

    def __init__(self, close_quote: str) -> None:
        self.close_quote: str = close_quote
        super().__init__(f'预计关闭 {close_quote}。')


class ExtensionError(QQException):
    """扩展相关错误的基本异常。

    这继承自 :exc:`~qq.QQException`。

    Attributes
    ------------
    name: :class:`str`
        有错误的扩展。
    """

    def __init__(self, message: Optional[str] = None, *args: Any, name: str) -> None:
        self.name: str = name
        message = message or f'扩展 {name!r} 有错误。'
        # clean-up @everyone and @here mentions
        m = message.replace('@everyone', '@\u200beveryone')
        super().__init__(m, *args)


class ExtensionAlreadyLoaded(ExtensionError):
    """已加载扩展时引发的异常。

    这继承自 :exc:`ExtensionError`
    """

    def __init__(self, name: str) -> None:
        super().__init__(f'扩展 {name!r} 已加载。', name=name)


class ExtensionNotLoaded(ExtensionError):
    """未加载扩展时引发的异常。

    这继承自 :exc:`ExtensionError`
    """

    def __init__(self, name: str) -> None:
        super().__init__(f'扩展 {name!r} 尚未加载。', name=name)


class NoEntryPointError(ExtensionError):
    """当扩展没有 ``setup`` 入口点函数时引发异常。

    这继承自 :exc:`ExtensionError`
    """

    def __init__(self, name: str) -> None:
        super().__init__(f"扩展 {name!r} 没有“setup” 函数。", name=name)


class ExtensionFailed(ExtensionError):
    """在执行模块或 ``setup`` 入口点期间无法加载扩展时引发的异常。

    这继承自 :exc:`ExtensionError`

    Attributes
    -----------
    name: :class:`str`
        出现错误的扩展。
    original: :exc:`Exception`
        引发的原始异常。 你也可以通过 ``__cause__`` 属性获取此信息。
    """

    def __init__(self, name: str, original: Exception) -> None:
        self.original: Exception = original
        msg = f'扩展 {name!r} 引发错误：{original.__class__.__name__}：{original}'
        super().__init__(msg, name=name)


class ExtensionNotFound(ExtensionError):
    """找不到扩展时引发的异常。

    这继承自 :exc:`ExtensionError`

    Attributes
    -----------
    name: :class:`str`
        出现错误的扩展。
    """

    def __init__(self, name: str) -> None:
        msg = f'无法加载扩展 {name!r}。'
        super().__init__(msg, name=name)


class CommandRegistrationError(ClientException):
    """由于名称已被其他命令采用而无法添加命令时引发异常。

    这继承自 :exc:`qq.ClientException`

    Attributes
    ----------
    name: :class:`str`
        出现错误的命令名称。
    alias_conflict: :class:`bool`
        冲突的名称是否是我们尝试添加的命令的别名。
    """

    def __init__(self, name: str, *, alias_conflict: bool = False) -> None:
        self.name: str = name
        self.alias_conflict: bool = alias_conflict
        type_ = 'alias' if alias_conflict else 'command'
        super().__init__(f'{type_} {name} 已经是一个现有的命令或别名。')


class FlagError(BadArgument):
    """所有标志解析相关错误的基本异常类型。

    这继承自 :exc:`BadArgument`。
    """
    pass


class TooManyFlags(FlagError):
    """当标志接收到太多值时引发异常。

    这继承自 :exc:`FlagError`。

    Attributes
    ------------
    flag: :class:`~qq.ext.commands.Flag`
        收到太多值的标志。
    values: List[:class:`str`]
        传递的值。
    """

    def __init__(self, flag: Flag, values: List[str]) -> None:
        self.flag: Flag = flag
        self.values: List[str] = values
        super().__init__(f'标志值太多，预期为 {flag.max_args}，但收到 {len(values)}。')


class BadFlagArgument(FlagError):
    """当标志无法转换值时引发异常。

    这继承自 :exc:`FlagError`。

    Attributes
    -----------
    flag: :class:`~qq.ext.commands.Flag`
        转换失败的标志。
    """

    def __init__(self, flag: Flag) -> None:
        self.flag: Flag = flag
        try:
            name = flag.annotation.__name__
        except AttributeError:
            name = flag.annotation.__class__.__name__

        super().__init__(f'无法为标志 {flag.name!r} 转换为 {name!r}')


class MissingRequiredFlag(FlagError):
    """未给出所需标志时引发异常。

    这继承自 :exc:`FlagError`

    Attributes
    -----------
    flag: :class:`~qq.ext.commands.Flag`
        未找到所需的标志。
    """

    def __init__(self, flag: Flag) -> None:
        self.flag: Flag = flag
        super().__init__(f'标记 {flag.name!r} 是必需的且缺失')


class MissingFlagArgument(FlagError):
    """标志未获得值时引发的异常。

    这继承自 :exc:`FlagError`

    Attributes
    -----------
    flag: :class:`~qq.ext.commands.Flag`
        未获得值的标志。
    """

    def __init__(self, flag: Flag) -> None:
        self.flag: Flag = flag
        super().__init__(f'标志 {flag.name!r} 没有参数')



import types
from collections import namedtuple
from typing import TYPE_CHECKING, ClassVar, Dict, Any, List, Type, TypeVar

__all__ = (
    'Enum',
    'ChannelType',
    'AudioStatusType'
)


def _create_value_cls(name, comparable):
    cls = namedtuple('_EnumValue_' + name, 'name value')
    cls.__repr__ = lambda self: f'<{name}.{self.name}: {self.value!r}>'
    cls.__str__ = lambda self: f'{name}.{self.name}'
    if comparable:
        cls.__le__ = lambda self, other: isinstance(other, self.__class__) and self.value <= other.value
        cls.__ge__ = lambda self, other: isinstance(other, self.__class__) and self.value >= other.value
        cls.__lt__ = lambda self, other: isinstance(other, self.__class__) and self.value < other.value
        cls.__gt__ = lambda self, other: isinstance(other, self.__class__) and self.value > other.value
    return cls


def _is_descriptor(obj):
    return hasattr(obj, '__get__') or hasattr(obj, '__set__') or hasattr(obj, '__delete__')


class EnumMeta(type):
    if TYPE_CHECKING:
        __name__: ClassVar[str]
        _enum_member_names_: ClassVar[List[str]]
        _enum_member_map_: ClassVar[Dict[str, Any]]
        _enum_value_map_: ClassVar[Dict[Any, Any]]

    def __new__(mcs, name, bases, attrs, *, comparable: bool = False):
        value_mapping = {}
        member_mapping = {}
        member_names = []

        value_cls = _create_value_cls(name, comparable)
        for key, value in list(attrs.items()):
            is_descriptor = _is_descriptor(value)
            if key[0] == '_' and not is_descriptor:
                continue

            # Special case classmethod to just pass through
            if isinstance(value, classmethod):
                continue

            if is_descriptor:
                setattr(value_cls, key, value)
                del attrs[key]
                continue

            try:
                new_value = value_mapping[value]
            except KeyError:
                new_value = value_cls(name=key, value=value)
                value_mapping[value] = new_value
                member_names.append(key)

            member_mapping[key] = new_value
            attrs[key] = new_value

        attrs['_enum_value_map_'] = value_mapping
        attrs['_enum_member_map_'] = member_mapping
        attrs['_enum_member_names_'] = member_names
        attrs['_enum_value_cls_'] = value_cls
        actual_cls = super().__new__(mcs, name, bases, attrs)
        value_cls._actual_enum_cls_ = actual_cls  # type: ignore
        return actual_cls

    def __iter__(cls):
        return (cls._enum_member_map_[name] for name in cls._enum_member_names_)

    def __reversed__(cls):
        return (cls._enum_member_map_[name] for name in reversed(cls._enum_member_names_))

    def __len__(cls):
        return len(cls._enum_member_names_)

    def __repr__(cls):
        return f'<enum {cls.__name__}>'

    @property
    def __members__(cls):
        return types.MappingProxyType(cls._enum_member_map_)

    def __call__(cls, value):
        try:
            return cls._enum_value_map_[value]
        except (KeyError, TypeError):
            raise ValueError(f"{value!r} is not a valid {cls.__name__}")

    def __getitem__(cls, key):
        return cls._enum_member_map_[key]

    def __setattr__(cls, name, value):
        raise TypeError('Enums are immutable.')

    def __delattr__(cls, attr):
        raise TypeError('Enums are immutable')

    def __instancecheck__(self, instance):
        # isinstance(x, Y)
        # -> __instancecheck__(Y, x)
        try:
            return instance._actual_enum_cls_ is self
        except AttributeError:
            return False


if TYPE_CHECKING:
    from enum import Enum
else:

    class Enum(metaclass=EnumMeta):
        @classmethod
        def try_value(cls, value):
            try:
                return cls._enum_value_map_[value]
            except (KeyError, TypeError):
                return value


class ChannelType(Enum):
    text = 0
    voice = 2
    category = 4
    live = 10005
    app = 10006
    thread = 10007

    def __str__(self):
        return self.name


class AudioStatusType(Enum):
    START = 0
    PAUSE = 1
    RESUME = 2
    STOP = 3

    def __str__(self):
        return self.name


T = TypeVar('T')


def create_unknown_value(cls: Type[T], val: Any) -> T:
    value_cls = cls._enum_value_cls_  # type: ignore
    name = f'unknown_{val}'
    return value_cls(name=name, value=val)


def try_enum(cls: Type[T], val: Any) -> T:
    """A function that tries to turn the value into enum ``cls``.
    If it fails it returns a proxy invalid value instead.
    """

    try:
        return cls._enum_value_map_[val]  # type: ignore
    except (KeyError, TypeError, AttributeError):
        return create_unknown_value(cls, val)



from __future__ import annotations

from typing import TYPE_CHECKING, Optional, Set, List

if TYPE_CHECKING:
    from .types.raw_models import (
        MessageDeleteEvent,
        ReactionActionEvent,
        ReactionClearEvent,
        ReactionClearEmojiEvent,
        IntegrationDeleteEvent
    )
    from .message import Message
    from .partial_emoji import PartialEmoji
    from .member import Member

__all__ = (
    'RawMessageDeleteEvent',
    'RawReactionActionEvent',
    'RawReactionClearEvent',
    'RawReactionClearEmojiEvent',
    'RawIntegrationDeleteEvent',
)


class _RawReprMixin:
    def __repr__(self) -> str:
        value = ' '.join(f'{attr}={getattr(self, attr)!r}' for attr in self.__slots__)
        return f'<{self.__class__.__name__} {value}>'


class RawMessageDeleteEvent(_RawReprMixin):
    """表示 :func:`on_raw_message_delete` 事件的事件负载。

    Attributes
    ------------
    channel_id: :class:`int`
        发生删除的子频道 ID。
    guild_id: Optional[:class:`int`]
        发生删除的频道 ID（如果适用）。
    message_id: :class:`int`
        被删除的消息 ID。
    cached_message: Optional[:class:`Message`]
        缓存的消息（如果在内部消息缓存中找到）。
    """

    __slots__ = ('message_id', 'channel_id', 'guild_id', 'cached_message')

    def __init__(self, data: MessageDeleteEvent) -> None:
        self.message_id: int = int(data['id'])
        self.channel_id: int = int(data['channel_id'])
        self.cached_message: Optional[Message] = None
        try:
            self.guild_id: Optional[int] = int(data['guild_id'])
        except KeyError:
            self.guild_id: Optional[int] = None


class RawReactionActionEvent(_RawReprMixin):
    """表示 :func:`on_raw_reaction_add` 或 :func:`on_raw_reaction_remove` 事件的负载。

    Attributes
    -----------
    message_id: :class:`int`
        得到或失去反应的消息 ID。
    user_id: :class:`int`
        添加反应或移除反应的用户 ID。
    channel_id: :class:`int`
        添加或删除反应的子频道 ID。
    guild_id: Optional[:class:`int`]
        添加或删除反应的频道 ID（如果适用）。
    emoji: :class:`PartialEmoji`
        正在使用的自定义或 unicode 表情符号。
    member: Optional[:class:`Member`]
        添加反应的成员。仅当 `event_type` 为 `REACTION_ADD` 且反应在频道内时可用。

    event_type: :class:`str`
        触发此操作的事件类型。可以是 
        ``REACTION_ADD`` 用于反应添加或 
        ``REACTION_REMOVE`` 用于反应去除。
    """

    __slots__ = ('message_id', 'user_id', 'channel_id', 'guild_id', 'emoji',
                 'event_type', 'member')

    def __init__(self, data: ReactionActionEvent, emoji: PartialEmoji, event_type: str) -> None:
        self.message_id: int = int(data['message_id'])
        self.channel_id: int = int(data['channel_id'])
        self.user_id: int = int(data['user_id'])
        self.emoji: PartialEmoji = emoji
        self.event_type: str = event_type
        self.member: Optional[Member] = None

        try:
            self.guild_id: Optional[int] = int(data['guild_id'])
        except KeyError:
            self.guild_id: Optional[int] = None


class RawReactionClearEvent(_RawReprMixin):
    """表示 :func:`on_raw_reaction_clear` 事件的负载。

    Attributes
    -----------
    message_id: :class:`int`
        清除其反应的消息 ID。
    channel_id: :class:`int`
        清除反应的子频道 ID。
    guild_id: Optional[:class:`int`]
        清除反应的频道 ID。
    """

    __slots__ = ('message_id', 'channel_id', 'guild_id')

    def __init__(self, data: ReactionClearEvent) -> None:
        self.message_id: int = int(data['message_id'])
        self.channel_id: int = int(data['channel_id'])

        try:
            self.guild_id: Optional[int] = int(data['guild_id'])
        except KeyError:
            self.guild_id: Optional[int] = None


class RawReactionClearEmojiEvent(_RawReprMixin):
    """表示 :func:`on_raw_reaction_clear_emoji` 事件的负载。

    Attributes
    -----------
    message_id: :class:`int`
        清除其反应的消息 ID。
    channel_id: :class:`int`
        清除反应的子频道 ID。
    guild_id: Optional[:class:`int`]
        清除反应的频道 ID。
    emoji: :class:`PartialEmoji`
        正在删除的自定义或 unicode 表情符号。
    """

    __slots__ = ('message_id', 'channel_id', 'guild_id', 'emoji')

    def __init__(self, data: ReactionClearEmojiEvent, emoji: PartialEmoji) -> None:
        self.emoji: PartialEmoji = emoji
        self.message_id: int = int(data['message_id'])
        self.channel_id: int = int(data['channel_id'])

        try:
            self.guild_id: Optional[int] = int(data['guild_id'])
        except KeyError:
            self.guild_id: Optional[int] = None


class RawIntegrationDeleteEvent(_RawReprMixin):
    """表示 :func:`on_raw_integration_delete` 事件的负载。

    Attributes
    -----------
    integration_id: :class:`int`
        被删除的集成的 ID。
    application_id: Optional[:class:`int`]
        此已删除集成的 botOAuth2 应用程序的 ID。
    guild_id: :class:`int`
        删除集成的频道 ID。
    """

    __slots__ = ('integration_id', 'application_id', 'guild_id')

    def __init__(self, data: IntegrationDeleteEvent) -> None:
        self.integration_id: int = int(data['id'])
        self.guild_id: int = int(data['guild_id'])

        try:
            self.application_id: Optional[int] = int(data['application_id'])
        except KeyError:
            self.application_id: Optional[int] = None



import colorsys
import random

from typing import (
    Any,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
)

__all__ = (
    'Colour',
    'Color',
)

CT = TypeVar('CT', bound='Colour')


class Colour:
    """
    代表 QQ 颜色。这个类类似于（红、绿、蓝）:class:`tuple` 。
    这个有一个别名叫做 Color。

    .. container:: operations

        .. describe:: x == y

             检查两种颜色是否相等。

        .. describe:: x != y

             检查两种颜色是否不相等。

        .. describe:: hash(x)

             返回颜色的哈希值。

        .. describe:: str(x)

             返回颜色的十六进制格式。

        .. describe:: int(x)

             返回原始颜色值。

    Attributes
    ------------
    value: :class:`int`
        原始整数颜色值。
    """

    __slots__ = ('value',)

    def __init__(self, value: int):
        if not isinstance(value, int):
            raise TypeError(f'Expected int parameter, received {value.__class__.__name__} instead.')

        self.value: int = value

    def _get_byte(self, byte: int) -> int:
        return (self.value >> (8 * byte)) & 0xff

    def __eq__(self, other: Any) -> bool:
        return isinstance(other, Colour) and self.value == other.value

    def __ne__(self, other: Any) -> bool:
        return not self.__eq__(other)

    def __str__(self) -> str:
        return f'#{self.value:0>6x}'

    def __int__(self) -> int:
        return self.value

    def __repr__(self) -> str:
        return f'<Colour value={self.value}>'

    def __hash__(self) -> int:
        return hash(self.value)

    @property
    def r(self) -> int:
        """:class:`int`: 返回颜色的红色部分。"""
        return self._get_byte(2)

    @property
    def g(self) -> int:
        """:class:`int`: 返回颜色的绿色部分。"""
        return self._get_byte(1)

    @property
    def b(self) -> int:
        """:class:`int`: 返回颜色的蓝色部分。"""
        return self._get_byte(0)

    def to_rgb(self) -> Tuple[int, int, int]:
        """Tuple[:class:`int`, :class:`int`, :class:`int`]: 返回表示颜色的 (r, g, b) 元组。"""
        return (self.r, self.g, self.b)

    @classmethod
    def from_rgb(cls: Type[CT], r: int, g: int, b: int) -> CT:
        """Constructs a :class:`Colour` 来自 RGB 元组。"""
        return cls((r << 16) + (g << 8) + b)

    @classmethod
    def from_hsv(cls: Type[CT], h: float, s: float, v: float) -> CT:
        """Constructs a :class:`Colour` 来自 HSV 元组。"""
        rgb = colorsys.hsv_to_rgb(h, s, v)
        return cls.from_rgb(*(int(x * 255) for x in rgb))

    @classmethod
    def default(cls: Type[CT]) -> CT:
        """一个工厂方法，它返回一个 :class:`Colour` 的值为 ``0`` 。"""
        return cls(0)

    @classmethod
    def random(cls: Type[CT], *, seed: Optional[Union[int, str, float, bytes, bytearray]] = None) -> CT:
        """返回带有随机色调的 :class:`Colour` 的工厂方法。

        .. note::

            随机算法的工作原理是选择具有随机色调但饱和度和值最大化的颜色。

        Parameters
        ------------
        seed: Optional[Union[:class:`int`, :class:`str`, :class:`float`, :class:`bytes`, :class:`bytearray`]]
            用于初始化随机数生成器的种子。如果传递 ``None``，则使用默认随机数生成器。

        """
        rand = random if seed is None else random.Random(seed)
        return cls.from_hsv(rand.random(), 1, 1)

    @classmethod
    def teal(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x1abc9c`` 的 :class:`Colour`  。"""
        return cls(0x1abc9c)

    @classmethod
    def dark_teal(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x11806a`` 的 :class:`Colour`  。"""
        return cls(0x11806a)

    @classmethod
    def brand_green(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x57F287`` 的 :class:`Colour`  。"""
        return cls(0x57F287)

    @classmethod
    def green(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x2ecc71`` 的 :class:`Colour`  。"""
        return cls(0x2ecc71)

    @classmethod
    def dark_green(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x1f8b4c`` 的 :class:`Colour`  。"""
        return cls(0x1f8b4c)

    @classmethod
    def blue(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x3498db`` 的 :class:`Colour`  。"""
        return cls(0x3498db)

    @classmethod
    def dark_blue(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x206694`` 的 :class:`Colour`  。"""
        return cls(0x206694)

    @classmethod
    def purple(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x9b59b6`` 的 :class:`Colour`  。"""
        return cls(0x9b59b6)

    @classmethod
    def dark_purple(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x71368a`` 的 :class:`Colour`  。"""
        return cls(0x71368a)

    @classmethod
    def magenta(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0xe91e63`` 的 :class:`Colour`  。"""
        return cls(0xe91e63)

    @classmethod
    def dark_magenta(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0xad1457`` 的 :class:`Colour`  。"""
        return cls(0xad1457)

    @classmethod
    def gold(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0xf1c40f`` 的 :class:`Colour`  。"""
        return cls(0xf1c40f)

    @classmethod
    def dark_gold(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0xc27c0e`` 的 :class:`Colour`  。"""
        return cls(0xc27c0e)

    @classmethod
    def orange(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0xe67e22`` 的 :class:`Colour`  。"""
        return cls(0xe67e22)

    @classmethod
    def dark_orange(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0xa84300`` 的 :class:`Colour`  。"""
        return cls(0xa84300)

    @classmethod
    def brand_red(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0xED4245`` 的 :class:`Colour`  。"""
        return cls(0xED4245)

    @classmethod
    def red(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0xe74c3c`` 的 :class:`Colour`  。"""
        return cls(0xe74c3c)

    @classmethod
    def dark_red(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x992d22`` 的 :class:`Colour`  。"""
        return cls(0x992d22)

    @classmethod
    def lighter_grey(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x95a5a6`` 的 :class:`Colour`  。"""
        return cls(0x95a5a6)

    lighter_gray = lighter_grey

    @classmethod
    def dark_grey(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x607d8b`` 的 :class:`Colour`  。"""
        return cls(0x607d8b)

    dark_gray = dark_grey

    @classmethod
    def light_grey(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x979c9f`` 的 :class:`Colour`  。"""
        return cls(0x979c9f)

    light_gray = light_grey

    @classmethod
    def darker_grey(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x546e7a`` 的 :class:`Colour`  。"""
        return cls(0x546e7a)

    darker_gray = darker_grey

    @classmethod
    def og_blurple(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x7289da`` 的 :class:`Colour`  。"""
        return cls(0x7289da)

    @classmethod
    def blurple(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x5865F2`` 的 :class:`Colour`  。"""
        return cls(0x5865F2)

    @classmethod
    def greyple(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x99aab5`` 的 :class:`Colour`  。"""
        return cls(0x99aab5)

    @classmethod
    def dark_theme(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0x36393F`` 的 :class:`Colour`  。"""
        return cls(0x36393F)

    @classmethod
    def fuchsia(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0xEB459E`` 的 :class:`Colour`  。"""
        return cls(0xEB459E)

    @classmethod
    def yellow(cls: Type[CT]) -> CT:
        """一个工厂方法，返回一个值为 ``0xFEE75C`` 的 :class:`Colour`  。"""
        return cls(0xFEE75C)


Color = Colour



from __future__ import annotations

import asyncio
import datetime
from typing import (
    Any,
    Awaitable,
    Callable,
    Generic,
    List,
    Optional,
    Type,
    TypeVar,
    Union,
)

import aiohttp
import inspect
import sys
import traceback
import qq
from collections.abc import Sequence
from qq.backoff import ExponentialBackoff
from qq.utils import MISSING

__all__ = (
    'loop',
)

T = TypeVar('T')
_func = Callable[..., Awaitable[Any]]
LF = TypeVar('LF', bound=_func)
FT = TypeVar('FT', bound=_func)
ET = TypeVar('ET', bound=Callable[[Any, BaseException], Awaitable[Any]])


class SleepHandle:
    __slots__ = ('future', 'loop', 'handle')

    def __init__(self, dt: datetime.datetime, *, loop: asyncio.AbstractEventLoop) -> None:
        self.loop = loop
        self.future = future = loop.create_future()
        relative_delta = qq.utils.compute_timedelta(dt)
        self.handle = loop.call_later(relative_delta, future.set_result, True)

    def recalculate(self, dt: datetime.datetime) -> None:
        self.handle.cancel()
        relative_delta = qq.utils.compute_timedelta(dt)
        self.handle = self.loop.call_later(relative_delta, self.future.set_result, True)

    def wait(self) -> asyncio.Future[Any]:
        return self.future

    def done(self) -> bool:
        return self.future.done()

    def cancel(self) -> None:
        self.handle.cancel()
        self.future.cancel()


class Loop(Generic[LF]):
    def __init__(
        self,
        coro: LF,
        seconds: float,
        hours: float,
        minutes: float,
        time: Union[datetime.time, Sequence[datetime.time]],
        count: Optional[int],
        reconnect: bool,
        loop: asyncio.AbstractEventLoop,
    ) -> None:
        self.coro: LF = coro
        self.reconnect: bool = reconnect
        self.loop: asyncio.AbstractEventLoop = loop
        self.count: Optional[int] = count
        self._current_loop = 0
        self._handle: SleepHandle = MISSING
        self._task: asyncio.Task[None] = MISSING
        self._injected = None
        self._valid_exception = (
            OSError,
            qq.GatewayNotFound,
            qq.ConnectionClosed,
            aiohttp.ClientError,
            asyncio.TimeoutError,
        )

        self._before_loop = None
        self._after_loop = None
        self._is_being_cancelled = False
        self._has_failed = False
        self._stop_next_iteration = False

        if self.count is not None and self.count <= 0:
            raise ValueError('计数必须大于 0 或无。')

        self.change_interval(seconds=seconds, minutes=minutes, hours=hours, time=time)
        self._last_iteration_failed = False
        self._last_iteration: datetime.datetime = MISSING
        self._next_iteration = None

        if not inspect.iscoroutinefunction(self.coro):
            raise TypeError(f'预期协程函数，而不是 {type(self.coro).__name__!r}。')

    async def _call_loop_function(self, name: str, *args: Any, **kwargs: Any) -> None:
        coro = getattr(self, '_' + name)
        if coro is None:
            return

        if self._injected is not None:
            await coro(self._injected, *args, **kwargs)
        else:
            await coro(*args, **kwargs)

    def _try_sleep_until(self, dt: datetime.datetime):
        self._handle = SleepHandle(dt=dt, loop=self.loop)
        return self._handle.wait()

    async def _loop(self, *args: Any, **kwargs: Any) -> None:
        backoff = ExponentialBackoff()
        await self._call_loop_function('before_loop')
        self._last_iteration_failed = False
        if self._time is not MISSING:
            # the time index should be prepared every time the internal loop is started
            self._prepare_time_index()
            self._next_iteration = self._get_next_sleep_time()
        else:
            self._next_iteration = datetime.datetime.now(datetime.timezone.utc)
        try:
            await self._try_sleep_until(self._next_iteration)
            while True:
                if not self._last_iteration_failed:
                    self._last_iteration = self._next_iteration
                    self._next_iteration = self._get_next_sleep_time()
                try:
                    await self.coro(*args, **kwargs)
                    self._last_iteration_failed = False
                except self._valid_exception:
                    self._last_iteration_failed = True
                    if not self.reconnect:
                        raise
                    await asyncio.sleep(backoff.delay())
                else:
                    await self._try_sleep_until(self._next_iteration)

                    if self._stop_next_iteration:
                        return

                    now = datetime.datetime.now(datetime.timezone.utc)
                    if now > self._next_iteration:
                        self._next_iteration = now
                        if self._time is not MISSING:
                            self._prepare_time_index(now)

                    self._current_loop += 1
                    if self._current_loop == self.count:
                        break

        except asyncio.CancelledError:
            self._is_being_cancelled = True
            raise
        except Exception as exc:
            self._has_failed = True
            await self._call_loop_function('error', exc)
            raise exc
        finally:
            await self._call_loop_function('after_loop')
            self._handle.cancel()
            self._is_being_cancelled = False
            self._current_loop = 0
            self._stop_next_iteration = False
            self._has_failed = False

    def __get__(self, obj: T, objtype: Type[T]) -> Loop[LF]:
        if obj is None:
            return self

        copy: Loop[LF] = Loop(
            self.coro,
            seconds=self._seconds,
            hours=self._hours,
            minutes=self._minutes,
            time=self._time,
            count=self.count,
            reconnect=self.reconnect,
            loop=self.loop,
        )
        copy._injected = obj
        copy._before_loop = self._before_loop
        copy._after_loop = self._after_loop
        copy._error = self._error
        setattr(obj, self.coro.__name__, copy)
        return copy

    @property
    def seconds(self) -> Optional[float]:
        if self._seconds is not MISSING:
            return self._seconds

    @property
    def minutes(self) -> Optional[float]:
        if self._minutes is not MISSING:
            return self._minutes

    @property
    def hours(self) -> Optional[float]:
        if self._hours is not MISSING:
            return self._hours

    @property
    def time(self) -> Optional[List[datetime.time]]:
        if self._time is not MISSING:
            return self._time.copy()

    @property
    def current_loop(self) -> int:
        return self._current_loop

    @property
    def next_iteration(self) -> Optional[datetime.datetime]:
        if self._task is MISSING:
            return None
        elif self._task and self._task.done() or self._stop_next_iteration:
            return None
        return self._next_iteration

    async def __call__(self, *args: Any, **kwargs: Any) -> Any:
        if self._injected is not None:
            args = (self._injected, *args)

        return await self.coro(*args, **kwargs)

    def start(self, *args: Any, **kwargs: Any) -> asyncio.Task[None]:
        if self._task is not MISSING and not self._task.done():
            raise RuntimeError('任务已启动且未完成。')

        if self._injected is not None:
            args = (self._injected, *args)

        if self.loop is MISSING:
            self.loop = asyncio.get_event_loop()

        self._task = self.loop.create_task(self._loop(*args, **kwargs))
        return self._task

    def stop(self) -> None:
        if self._task is not MISSING and not self._task.done():
            self._stop_next_iteration = True

    def _can_be_cancelled(self) -> bool:
        return bool(not self._is_being_cancelled and self._task and not self._task.done())

    def cancel(self) -> None:
        if self._can_be_cancelled():
            self._task.cancel()

    def restart(self, *args: Any, **kwargs: Any) -> None:
        def restart_when_over(fut: Any, *, args: Any = args, kwargs: Any = kwargs) -> None:
            self._task.remove_done_callback(restart_when_over)
            self.start(*args, **kwargs)

        if self._can_be_cancelled():
            self._task.add_done_callback(restart_when_over)
            self._task.cancel()

    def add_exception_type(self, *exceptions: Type[BaseException]) -> None:
        for exc in exceptions:
            if not inspect.isclass(exc):
                raise TypeError(f'{exc!r} 必须是一个类。')
            if not issubclass(exc, BaseException):
                raise TypeError(f'{exc!r} 必须从 BaseException 继承。')

        self._valid_exception = (*self._valid_exception, *exceptions)

    def clear_exception_types(self) -> None:
        self._valid_exception = tuple()

    def remove_exception_type(self, *exceptions: Type[BaseException]) -> bool:
        old_length = len(self._valid_exception)
        self._valid_exception = tuple(x for x in self._valid_exception if x not in exceptions)
        return len(self._valid_exception) == old_length - len(exceptions)

    def get_task(self) -> Optional[asyncio.Task[None]]:
        return self._task if self._task is not MISSING else None

    def is_being_cancelled(self) -> bool:
        return self._is_being_cancelled

    def failed(self) -> bool:
        return self._has_failed

    def is_running(self) -> bool:
        return not bool(self._task.done()) if self._task is not MISSING else False

    async def _error(self, *args: Any) -> None:
        exception: Exception = args[-1]
        print(f'内部后台任务 {self.coro.__name__!r} 中未处理的异常。', file=sys.stderr)
        traceback.print_exception(type(exception), exception, exception.__traceback__, file=sys.stderr)

    def before_loop(self, coro: FT) -> FT:
        if not inspect.iscoroutinefunction(coro):
            raise TypeError(f'预期协程函数，收到 {coro.__class__.__name__!r}。')

        self._before_loop = coro
        return coro

    def after_loop(self, coro: FT) -> FT:
        if not inspect.iscoroutinefunction(coro):
            raise TypeError(f'预期协程函数，收到 {coro.__class__.__name__!r}。')

        self._after_loop = coro
        return coro

    def error(self, coro: ET) -> ET:
        if not inspect.iscoroutinefunction(coro):
            raise TypeError(f'预期协程函数，收到 {coro.__class__.__name__!r}。')

        self._error = coro  # type: ignore
        return coro

    def _get_next_sleep_time(self) -> datetime.datetime:
        if self._sleep is not MISSING:
            return self._last_iteration + datetime.timedelta(seconds=self._sleep)

        if self._time_index >= len(self._time):
            self._time_index = 0
            if self._current_loop == 0:
                # if we're at the last index on the first iteration, we need to sleep until tomorrow
                return datetime.datetime.combine(
                    datetime.datetime.now(datetime.timezone.utc) + datetime.timedelta(days=1), self._time[0]
                )

        next_time = self._time[self._time_index]

        if self._current_loop == 0:
            self._time_index += 1
            return datetime.datetime.combine(datetime.datetime.now(datetime.timezone.utc), next_time)

        next_date = self._last_iteration
        if self._time_index == 0:
            # we can assume that the earliest time should be scheduled for "tomorrow"
            next_date += datetime.timedelta(days=1)

        self._time_index += 1
        return datetime.datetime.combine(next_date, next_time)

    def _prepare_time_index(self, now: datetime.datetime = MISSING) -> None:
        # now kwarg should be a datetime.datetime representing the time "now"
        # to calculate the next time index from

        # pre-condition: self._time is set
        time_now = (
            now if now is not MISSING else datetime.datetime.now(datetime.timezone.utc).replace(microsecond=0)
        ).timetz()
        for idx, time in enumerate(self._time):
            if time >= time_now:
                self._time_index = idx
                break
        else:
            self._time_index = 0

    def _get_time_parameter(
        self,
        time: Union[datetime.time, Sequence[datetime.time]],
        *,
        dt: Type[datetime.time] = datetime.time,
        utc: datetime.timezone = datetime.timezone.utc,
    ) -> List[datetime.time]:
        if isinstance(time, dt):
            inner = time if time.tzinfo is not None else time.replace(tzinfo=utc)
            return [inner]
        if not isinstance(time, Sequence):
            raise TypeError(
                f'预期 ``datetime.time`` 或 ``time`` 的一系列 ``datetime.time`` ，而是收到 {type(time)!r}。'
            )
        if not time:
            raise ValueError('时间参数不能是空序列。')

        ret: List[datetime.time] = []
        for index, t in enumerate(time):
            if not isinstance(t, dt):
                raise TypeError(
                    f'预期 ``time`` 的 {dt!r} 序列，而是在索引 {index} 处收到 {type(t).__name__!r} 。'
                )
            ret.append(t if t.tzinfo is not None else t.replace(tzinfo=utc))

        ret = sorted(set(ret))  # de-dupe and sort times
        return ret

    def change_interval(
        self,
        *,
        seconds: float = 0,
        minutes: float = 0,
        hours: float = 0,
        time: Union[datetime.time, Sequence[datetime.time]] = MISSING,
    ) -> None:
        if time is MISSING:
            seconds = seconds or 0
            minutes = minutes or 0
            hours = hours or 0
            sleep = seconds + (minutes * 60.0) + (hours * 3600.0)
            if sleep < 0:
                raise ValueError('总秒数不能小于零。')

            self._sleep = sleep
            self._seconds = float(seconds)
            self._hours = float(hours)
            self._minutes = float(minutes)
            self._time: List[datetime.time] = MISSING
        else:
            if any((seconds, minutes, hours)):
                raise TypeError('不能将显式时间与相对时间混合')
            self._time = self._get_time_parameter(time)
            self._sleep = self._seconds = self._minutes = self._hours = MISSING

        if self.is_running():
            if self._time is not MISSING:
                # prepare the next time index starting from after the last iteration
                self._prepare_time_index(now=self._last_iteration)

            self._next_iteration = self._get_next_sleep_time()
            if not self._handle.done():
                # the loop is sleeping, recalculate based on new interval
                self._handle.recalculate(self._next_iteration)


def loop(
    *,
    seconds: float = MISSING,
    minutes: float = MISSING,
    hours: float = MISSING,
    time: Union[datetime.time, Sequence[datetime.time]] = MISSING,
    count: Optional[int] = None,
    reconnect: bool = True,
    loop: asyncio.AbstractEventLoop = MISSING,
) -> Callable[[LF], Loop[LF]]:
    """使用可选的重新连接逻辑在后台为您安排任务的装饰器。装饰器返回一个:class:`Loop`。

    Parameters
    ------------
    seconds: :class:`float`
        每次迭代之间的秒数。
    minutes: :class:`float`
        每次迭代之间的分钟数。
    hours: :class:`float`
        每次迭代之间的小时数。
    time: Union[:class:`datetime.time`, Sequence[:class:`datetime.time`]]
        运行此循环的确切时间。应该传递一个非空列表或一个 :class:`datetime.time` 的值。支持时区。
        如果没有给出时间的时区，则假定它代表 UTC 时间。这不能与相对时间参数结合使用。

        .. note::

            重复时间将被忽略，并且只运行一次。

    count: Optional[:class:`int`]
        要执行的循环次数，如果它应该是无限循环，则为“无”。
    reconnect: :class:`bool`
        是否使用类似于 ``qq.Client.connect`` 中使用的指数退避算法处理错误并重新启动任务。
    loop: :class:`asyncio.AbstractEventLoop`
        用于注册任务的循环，如果没有给出，则默认为 :func:`asyncio.get_event_loop`。

    Raises
    --------
    ValueError
        给出了无效值。
    TypeError
        该函数不是协程，传递的 ``time`` 参数的值无效，或者 ``time`` 参数与相对时间参数一起传递。
    """

    def decorator(func: LF) -> Loop[LF]:
        return Loop[LF](
            func,
            seconds=seconds,
            minutes=minutes,
            hours=hours,
            count=count,
            time=time,
            reconnect=reconnect,
            loop=loop,
        )

    return decorator



__path__ = __import__('pkgutil').extend_path(__path__, __name__)
__title__ = 'qq'
__author__ = 'Foxwhite'
__license__ = 'MIT'
__version__ = '1.0.5'

import logging
from typing import NamedTuple, Literal
from .client import *
from .user import *
from .channel import *
from .guild import *
from .flags import *
from .member import *
from .message import *
from .asset import *
from .role import *
from .file import *
from .colour import *
from .object import *
from . import utils, abc
from .enum import *
from .shard import *
from .error import *
from .embeds import *
from .mention import *
from .partial_emoji import *
from .raw_models import *


class VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: Literal["alpha", "beta", "candidate", "final"]
    serial: int


version_info: VersionInfo = VersionInfo(major=1, minor=0, micro=5, releaselevel='beta', serial=0)
